'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var preact = require('preact');
var undom = _interopDefault(require('undom'));

var VOID_ELEMENTS = [
  'area',
  'base',
  'br',
  'col',
  'embed',
  'hr',
  'img',
  'input',
  'link',
  'meta',
  'param',
  'source',
  'track',
  'wbr'
];

var ESC = {
  '&': 'amp',
  '<': 'lt',
  '>': 'gt',
  '"': 'quot',
  "'": 'apos'
};

var encode = function (s) { return s.replace(/[&'"<>]/g, function (a) { return ("&" + (ESC[a])); }); };

var attr = function (a) {
  if (a.name === 'class' && a.value === '') {
    return ''
  }
  return (" " + (a.name.replace(/^html/, '')) + (a.value === 'true' || a.value === '' ? '' : ("=\"" + (encode(a.value)) + "\"")))
};

var serializeHtml = function (el) {
  var nodeType = el.nodeType;
  var nodeName = el.nodeName;
  var textContent = el.textContent;
  var attributes = el.attributes;
  var childNodes = el.childNodes;
  var innerHTML = el.innerHTML;
  var normalizedNodeName = nodeName.toLowerCase();
  if (nodeType === 3) {
    return encode(textContent)
  }
  var start = "<" + normalizedNodeName + (attributes.map(attr).join(''));
  if (VOID_ELEMENTS.includes(normalizedNodeName)) {
    return (start + " />")
  }
  return (start + ">" + (innerHTML || childNodes.map(serializeHtml).join('')) + "</" + normalizedNodeName + ">")
};

var doc;
var createRenderer = function () {
  if (!doc) {
    doc = undom();
    Object.assign(global, doc.defaultView);
  }

  var root;
  var parent = doc.createElement('x-root');
  doc.body.appendChild(parent);

  var instance = {  // todo: promisify this
    render: function (jsx) {
      root = preact.render(jsx, parent, root);
      return instance
    },
    toHtml: function () { return serializeHtml(root); },
    tearDown: function () { return preact.render(React.createElement( 'nothing', null ), parent, root).remove(); }
  };
  return instance
};

exports.createRenderer = createRenderer;
